==> emelfm2-0.1.2.tar.gz: emelfm2-0.1.2/src/dialogs/e2_ownership_dialog.c <==
/* $Id: e2_ownership_dialog.c 222 2005-01-22 07:03:24Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
//#include <unistd.h>
#include <pwd.h>
#include <grp.h>
#include "e2_dialog.h"
#include "e2_ownership_dialog.h"

static GtkWidget *user_combo;
static GtkWidget *group_combo;
static GtkWidget *recurse_dirs_button;
static uid_t *owner;
static gid_t *group;
static gboolean *recurse;
static gboolean *permission;

static uid_t _e2_ownership_dialog_get_user_id (void)
{
  gchar *user;
  struct passwd *pw_buf;
  uid_t user_id;
  
  user = (gchar *) gtk_entry_get_text (GTK_ENTRY (GTK_BIN (user_combo)->child));
  if ((pw_buf = getpwnam (user)) != NULL)
    user_id = pw_buf->pw_uid;
  else
    user_id = (uid_t)atoi (user);

  return user_id;
}

static gid_t _e2_ownership_dialog_get_group_id (void)
{
  gchar *group;
  struct group *grp_buf;
  gid_t group_id;

  group = (gchar *) gtk_entry_get_text (GTK_ENTRY (GTK_BIN (group_combo)->child));
  if ((grp_buf = getgrnam (group)) != NULL)
    group_id = grp_buf->gr_gid;
  else
    group_id = (gid_t)atoi (group);

  return group_id;
}

static void _e2_ownership_dialog_ok_cb (GtkWidget *widget)
{
  *owner = _e2_ownership_dialog_get_user_id ();
  *group = _e2_ownership_dialog_get_group_id ();
  return;
}

static void _e2_ownership_dialog_toggle_recurse_button_cb (GtkWidget *widget)
{
  if (*permission)
	*recurse = GTK_TOGGLE_BUTTON (recurse_dirs_button)->active;
  return;
}

void e2_ownership_dialog_warn (GtkWidget *box)
{
	GtkWidget *frame = gtk_frame_new ("");
	gtk_box_pack_start (GTK_BOX (box), frame, TRUE, TRUE, E2_PADDING);
	gtk_widget_show (frame);

	GtkWidget *sub_vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (frame), sub_vbox);
  	gtk_widget_show (sub_vbox);
	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (sub_vbox), hbox);
	e2_widget_add_label (hbox, _("You are not authorised to make any change."), 0.5,
		TRUE, E2_PADDING*2);
	gtk_widget_show (hbox);
	//padding line
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (sub_vbox), hbox);
    e2_widget_add_label (hbox, (" "), 0, FALSE, 0);
  	gtk_widget_show (hbox);
}

GtkWidget *e2_ownership_dialog_setup (E2_SelectedItemInfo *info,
	uid_t *owner_ret, gid_t *group_ret, gboolean *recurse_ret, 
	gboolean *permission_ret, gboolean multisrc)
{
  GtkWidget *ownership_dialog;
  GtkWidget *dialog_vbox;
  GtkWidget *action_area;
  GtkWidget *table, *hbox;
  GString *label_text = g_string_sized_new (NAME_MAX+20);
  gchar perm_string[32];
  struct stat statbuf;

//  gchar *itempath = g_strconcat (curr_view->dir, info->filename, NULL);
// e2_fs_e2_fs_lstat (itempath, &statbuf);
//  g_free (itempath);
  //path is to current dir
  F_DECLARE;
  gchar *local = F_FILENAME_TO_LOCALE (info->filename);
  e2_fs_lstat (local, &statbuf);
  F_FREE (local);
  // Set up pointers to static parameters
  owner = owner_ret;
  group = group_ret;
  recurse = recurse_ret;
  permission = permission_ret;
	
  ownership_dialog = gtk_dialog_new ();
  E2_DIALOG_TITLE (GTK_WINDOW (ownership_dialog), _("ownership"));
  dialog_vbox = GTK_DIALOG (ownership_dialog)->vbox;
  action_area = GTK_DIALOG (ownership_dialog)->action_area; //this is for the buttons
  gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);

//  table = add_table (dialog_vbox, 2, 3, TRUE, TRUE, 0); //2 rows, 3 cols, homogen, fill, no pad
//  gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
  gboolean thisis_dir = e2_fs_is_dir2 (info);
  gchar *label = (thisis_dir) ? _("Directory name") : _("Filename") ;
  g_string_printf (label_text, "%s: <b>%s</b>", label, info->filename);
  //add_label_to_table (table, label_text->str, 0, 1, 2, 0, 1);
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, E2_PADDING); //top, bottom padding 
  e2_widget_add_label (hbox, label_text->str, 0, TRUE, E2_PADDING); //L, R padding
  gtk_widget_show (hbox);

  e2_fs_get_perm_string (perm_string, sizeof (perm_string), statbuf.st_mode);
  g_string_assign (label_text, perm_string);
  //FIXME this assumes 1-byte chars for permission codes
  g_string_insert_c (label_text, 7, ' ');
  g_string_insert_c (label_text, 4, ' ');
  g_string_insert_c (label_text, 1, ' ');
  g_string_prepend (label_text, _("Permissions: "));
//  add_label_to_table (table, label_text->str, 0, 1, 2, 1, 2); 
  hbox = gtk_hbox_new (FALSE, 0);
  e2_widget_add_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
  gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show (hbox);
  
  g_string_free (label_text, TRUE);
  
  table = add_table (dialog_vbox, 2, 3, TRUE, TRUE, 0);  //2 rows, 3 cols, homogen, fill, no pad
  gtk_container_set_border_width (GTK_CONTAINER (table), E2_PADDING);
  gtk_table_set_row_spacings (GTK_TABLE (table), E2_PADDING);
  add_label_to_table (table, _("User: "), 0, 0, 1, 0, 1);

  user_combo = gtk_combo_box_entry_new_text ();  //note that this has no vertical scoll bar or equivalent !!
  gtk_table_attach_defaults (GTK_TABLE (table), user_combo, 1, 3, 0, 1);  // gint left col, gint right col, gint top row, gint bottom row)
  gtk_widget_show (user_combo);

  add_label_to_table (table, _("Group: "), 0, 0, 1, 1, 2);

  group_combo = gtk_combo_box_entry_new_text ();
  gtk_table_attach_defaults (GTK_TABLE (table), group_combo, 1, 3, 1, 2);
  gtk_widget_show (group_combo);

  {  //setup the combobox data
    struct passwd *pw_buf;
    struct group *grp_buf;
    gchar id_buf[32];
	gint ctr = 0;	//combo box entry counter, set to 1st value in list, for now
	gint user_index = 0, grp_index = 0;  //default values for combobox display (= 1st entry)

	gint src_uid = (gint) statbuf.st_uid;
	gint myuid = getuid ();
	setpwent ();

//	gchar *itempath = g_strconcat (curr_view->dir, info->filename, NULL);
//	gboolean writeok = e2_utils_check_write_permission (myuid, itempath);
//	g_free (itempath);
	//path is to current dir
	gboolean writeok = e2_fs_check_write_permission (myuid, info->filename);
	if (writeok)
    {
		*permission=TRUE;	//confirm ok to proceed
		while ((pw_buf = getpwent ()) != NULL)
	   {	//screen out superfluous names
			if (
				(myuid > 0) //user is not root
				&& ((gint) pw_buf->pw_uid > 0)  //buffer entry is not root's
				&& ((gint) pw_buf->pw_uid < UID_LOWLIMIT)  //but is less than the system-user threshold
			) 
				continue;
			gtk_combo_box_append_text (GTK_COMBO_BOX (user_combo), g_strdup (pw_buf->pw_name));
			//remember the combo index for the owner
			if ((gint) pw_buf->pw_uid == src_uid)
				user_index = ctr;
		   else
			   ctr++;
		}
		if ((pw_buf = getpwuid (statbuf.st_uid)) == NULL)
		{
			g_snprintf (id_buf, sizeof (id_buf), "%d", (gint) statbuf.st_uid);
			gtk_combo_box_append_text (GTK_COMBO_BOX (user_combo), id_buf);
			user_index = ctr;
		}
		printd (DEBUG, "owner combo index = %d", user_index);

		ctr = 0;
		gint src_gid = (gint) statbuf.st_gid;
		if (myuid == 0)
		{
			while ((grp_buf = getgrent ()) != NULL)
			{
			  gtk_combo_box_append_text (GTK_COMBO_BOX (group_combo), g_strdup (grp_buf->gr_name));
			  if ((gint) grp_buf->gr_gid == src_gid)
				grp_index = ctr;
			  else
				ctr++;
			}
		}
		else //not root
		{
			gid_t grp_ids[REPORTABLE_GROUPS];
			gint n = getgroups (REPORTABLE_GROUPS, grp_ids);
			for (ctr = 0; ctr < n; ctr++)
			{
				if ((grp_buf = getgrgid (grp_ids[ctr])) != NULL)
					gtk_combo_box_append_text (GTK_COMBO_BOX (group_combo), g_strdup (grp_buf->gr_name));
				if ((gint) grp_buf->gr_gid == src_gid)
					grp_index = ctr;
			}
		}
		if ((grp_buf = getgrgid (statbuf.st_gid)) == NULL)
		{
			g_snprintf (id_buf, sizeof (id_buf), "%d", (gint) statbuf.st_gid);
			gtk_combo_box_append_text (GTK_COMBO_BOX (group_combo), id_buf);
			grp_index = ctr;
		}
		printd (DEBUG, "group combo index = %d", grp_index);
	}
	else  //no write permission, no changes allowed, just show current values
    {
		*permission=FALSE;	//signal not ok to proceed, even if ok is clicked
    	if ((pw_buf = getpwuid (statbuf.st_uid)) != NULL)
	    	gtk_combo_box_append_text (GTK_COMBO_BOX (user_combo), g_strdup (pw_buf->pw_name));
		else
	 	{
			g_snprintf (id_buf, sizeof (id_buf), "%d", (gint) statbuf.st_uid);
			gtk_combo_box_append_text (GTK_COMBO_BOX (user_combo), id_buf);
		}
    	if ((grp_buf = getgrgid (statbuf.st_gid)) != NULL)
			gtk_combo_box_append_text (GTK_COMBO_BOX (group_combo), g_strdup (grp_buf->gr_name));
		else
		{
			g_snprintf (id_buf, sizeof (id_buf), "%d", (gint) statbuf.st_gid);
			gtk_combo_box_append_text (GTK_COMBO_BOX (group_combo), id_buf);
		}
		gtk_widget_set_sensitive (user_combo, FALSE);
		gtk_widget_set_sensitive (group_combo, FALSE);
	}
    gtk_combo_box_set_active (GTK_COMBO_BOX (user_combo), user_index);
    gtk_combo_box_set_active (GTK_COMBO_BOX (group_combo), grp_index);

    setpwent ();
    setgrent ();
  }

  if (*permission && thisis_dir)  //note: no recurse button now if multiple-selection
  {
 	 table = add_table (dialog_vbox, 1, 3, TRUE, TRUE, 0); //1 row, 3 cols, homogen, fill, no pad
  	gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
  	recurse_dirs_button = e2_button_add_toggle_to_table (table, _("Recurse"), 
	  FALSE, _e2_ownership_dialog_toggle_recurse_button_cb,
	  NULL, 1, 2, 0, 1 ); // gint left, gint right, gint top, gint bottom
  }

  if (!*permission)
	e2_ownership_dialog_warn (dialog_vbox);
  
// add buttons in the order that they will appear
  if (multisrc)
	{
		e2_dialog_add_defined_button (ownership_dialog, &E2_BUTTON_NOTOALL);
		if (*permission)
			e2_dialog_add_button_custom ( ownership_dialog, FALSE, &E2_BUTTON_APPLYTOALL,
			NULL, _e2_ownership_dialog_ok_cb, NULL);
	}
	if (*permission)
		e2_dialog_add_defined_button (ownership_dialog, &E2_BUTTON_CANCEL);
	e2_dialog_add_button_custom (ownership_dialog, TRUE, &E2_BUTTON_OK,
	NULL, _e2_ownership_dialog_ok_cb, NULL);

	e2_dialog_setup (ownership_dialog, app.main_window);
	return ownership_dialog;
}

==> emelfm2-0.1.4.tar.gz: emelfm2-0.1.4/src/dialogs/e2_ownership_dialog.c <==
/* $Id: e2_ownership_dialog.c 222 2005-01-22 07:03:24Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
//#include <unistd.h>
#include <pwd.h>
#include <grp.h>
#include "e2_dialog.h"
#include "e2_ownership_dialog.h"

static GtkWidget *user_combo;
static GtkWidget *group_combo;
static GtkWidget *recurse_dirs_button;
static uid_t *owner;
static gid_t *group;
static gboolean *recurse;
static gboolean *permission;

static uid_t _e2_ownership_dialog_get_user_id (void)
{
  gchar *user;
  struct passwd *pw_buf;
  uid_t user_id;
  
  user = (gchar *) gtk_entry_get_text (GTK_ENTRY (GTK_BIN (user_combo)->child));
  if ((pw_buf = getpwnam (user)) != NULL)
    user_id = pw_buf->pw_uid;
  else
    user_id = (uid_t)atoi (user);

  return user_id;
}

static gid_t _e2_ownership_dialog_get_group_id (void)
{
  gchar *group;
  struct group *grp_buf;
  gid_t group_id;

  group = (gchar *) gtk_entry_get_text (GTK_ENTRY (GTK_BIN (group_combo)->child));
  if ((grp_buf = getgrnam (group)) != NULL)
    group_id = grp_buf->gr_gid;
  else
    group_id = (gid_t)atoi (group);

  return group_id;
}

static void _e2_ownership_dialog_ok_cb (GtkWidget *widget)
{
  *owner = _e2_ownership_dialog_get_user_id ();
  *group = _e2_ownership_dialog_get_group_id ();
  return;
}

static void _e2_ownership_dialog_toggle_recurse_button_cb (GtkWidget *widget)
{
  if (*permission)
	*recurse = GTK_TOGGLE_BUTTON (recurse_dirs_button)->active;
  return;
}

void e2_ownership_dialog_warn (GtkWidget *box)
{
	GtkWidget *frame = gtk_frame_new ("");
	gtk_box_pack_start (GTK_BOX (box), frame, TRUE, TRUE, E2_PADDING);
	gtk_widget_show (frame);

	GtkWidget *sub_vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (frame), sub_vbox);
  	gtk_widget_show (sub_vbox);
	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (sub_vbox), hbox);
	e2_widget_add_label (hbox, _("You are not authorised to make any change."), 0.5,
		TRUE, E2_PADDING*2);
	gtk_widget_show (hbox);
	//padding line
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (sub_vbox), hbox);
    e2_widget_add_label (hbox, (" "), 0, FALSE, 0);
  	gtk_widget_show (hbox);
}

GtkWidget *e2_ownership_dialog_setup (E2_SelectedItemInfo *info,
	uid_t *owner_ret, gid_t *group_ret, gboolean *recurse_ret, 
	gboolean *permission_ret, gboolean multisrc)
{
  GtkWidget *ownership_dialog;
  GtkWidget *dialog_vbox;
  GtkWidget *action_area;
  GtkWidget *table, *hbox;
  GString *label_text = g_string_sized_new (NAME_MAX+20);
  gchar perm_string[32];
  struct stat statbuf;

//  gchar *itempath = g_strconcat (curr_view->dir, info->filename, NULL);
// e2_fs_e2_fs_lstat (itempath, &statbuf);
//  g_free (itempath);
  //path is to current dir
  gchar *local = F_FILENAME_TO_LOCALE (info->filename);
  e2_fs_lstat (local, &statbuf);
  F_FREE (local);
  // Set up pointers to static parameters
  owner = owner_ret;
  group = group_ret;
  recurse = recurse_ret;
  permission = permission_ret;
	
  ownership_dialog = gtk_dialog_new ();
  E2_DIALOG_TITLE (GTK_WINDOW (ownership_dialog), _("ownership"));
  dialog_vbox = GTK_DIALOG (ownership_dialog)->vbox;
  action_area = GTK_DIALOG (ownership_dialog)->action_area; //this is for the buttons
  gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);

//  table = add_table (dialog_vbox, 2, 3, TRUE, TRUE, 0); //2 rows, 3 cols, homogen, fill, no pad
//  gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
  gboolean thisis_dir = e2_fs_is_dir2 (info);
  gchar *label = (thisis_dir) ? _("Directory name") : _("Filename") ;
  g_string_printf (label_text, "%s: <b>%s</b>", label, info->filename);
  //add_label_to_table (table, label_text->str, 0, 1, 2, 0, 1);
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, E2_PADDING); //top, bottom padding 
  e2_widget_add_label (hbox, label_text->str, 0, TRUE, E2_PADDING); //L, R padding
  gtk_widget_show (hbox);

  e2_fs_get_perm_string (perm_string, sizeof (perm_string), statbuf.st_mode);
  g_string_assign (label_text, perm_string);
  //FIXME this assumes 1-byte chars for permission codes
  g_string_insert_c (label_text, 7, ' ');
  g_string_insert_c (label_text, 4, ' ');
  g_string_insert_c (label_text, 1, ' ');
  g_string_prepend (label_text, _("Permissions: "));
//  add_label_to_table (table, label_text->str, 0, 1, 2, 1, 2); 
  hbox = gtk_hbox_new (FALSE, 0);
  e2_widget_add_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
  gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show (hbox);
  
  g_string_free (label_text, TRUE);
  
  table = add_table (dialog_vbox, 2, 3, TRUE, TRUE, 0);  //2 rows, 3 cols, homogen, fill, no pad
  gtk_container_set_border_width (GTK_CONTAINER (table), E2_PADDING);
  gtk_table_set_row_spacings (GTK_TABLE (table), E2_PADDING);
  add_label_to_table (table, _("User: "), 0, 0, 1, 0, 1);

  user_combo = gtk_combo_box_entry_new_text ();  //note that this has no vertical scoll bar or equivalent !!
  gtk_table_attach_defaults (GTK_TABLE (table), user_combo, 1, 3, 0, 1);  // gint left col, gint right col, gint top row, gint bottom row)
  gtk_widget_show (user_combo);

  add_label_to_table (table, _("Group: "), 0, 0, 1, 1, 2);

  group_combo = gtk_combo_box_entry_new_text ();
  gtk_table_attach_defaults (GTK_TABLE (table), group_combo, 1, 3, 1, 2);
  gtk_widget_show (group_combo);

  {  //setup the combobox data
    struct passwd *pw_buf;
    struct group *grp_buf;
    gchar id_buf[32];
	gint ctr = 0;	//combo box entry counter, set to 1st value in list, for now
	gint user_index = 0, grp_index = 0;  //default values for combobox display (= 1st entry)

	gint src_uid = (gint) statbuf.st_uid;
	gint myuid = getuid ();
	setpwent ();

//	gchar *itempath = g_strconcat (curr_view->dir, info->filename, NULL);
//	gboolean writeok = e2_utils_check_write_permission (myuid, itempath);
//	g_free (itempath);
	//path is to current dir
	gboolean writeok = e2_fs_check_write_permission (myuid, info->filename);
	if (writeok)
    {
		*permission=TRUE;	//confirm ok to proceed
		while ((pw_buf = getpwent ()) != NULL)
	   {	//screen out superfluous names
			if (
				(myuid > 0) //user is not root
				&& ((gint) pw_buf->pw_uid > 0)  //buffer entry is not root's
				&& ((gint) pw_buf->pw_uid < UID_LOWLIMIT)  //but is less than the system-user threshold
			) 
				continue;
			gtk_combo_box_append_text (GTK_COMBO_BOX (user_combo), g_strdup (pw_buf->pw_name));
			//remember the combo index for the owner
			if ((gint) pw_buf->pw_uid == src_uid)
				user_index = ctr;
		   else
			   ctr++;
		}
		if ((pw_buf = getpwuid (statbuf.st_uid)) == NULL)
		{
			g_snprintf (id_buf, sizeof (id_buf), "%d", (guint) statbuf.st_uid);
			gtk_combo_box_append_text (GTK_COMBO_BOX (user_combo), id_buf);
			user_index = ctr;
		}
		printd (DEBUG, "owner combo index = %d", user_index);

		ctr = 0;
		gint src_gid = (gint) statbuf.st_gid;
		if (myuid == 0)
		{
			while ((grp_buf = getgrent ()) != NULL)
			{
			  gtk_combo_box_append_text (GTK_COMBO_BOX (group_combo), g_strdup (grp_buf->gr_name));
			  if ((gint) grp_buf->gr_gid == src_gid)
				grp_index = ctr;
			  else
				ctr++;
			}
		}
		else //not root
		{
			gid_t grp_ids[REPORTABLE_GROUPS];
			gint n = getgroups (REPORTABLE_GROUPS, grp_ids);
			for (ctr = 0; ctr < n; ctr++)
			{
				if ((grp_buf = getgrgid (grp_ids[ctr])) != NULL)
					gtk_combo_box_append_text (GTK_COMBO_BOX (group_combo), g_strdup (grp_buf->gr_name));
				if ((gint) grp_buf->gr_gid == src_gid)
					grp_index = ctr;
			}
		}
		if ((grp_buf = getgrgid (statbuf.st_gid)) == NULL)
		{
			g_snprintf (id_buf, sizeof (id_buf), "%d", (guint) statbuf.st_gid);
			gtk_combo_box_append_text (GTK_COMBO_BOX (group_combo), id_buf);
			grp_index = ctr;
		}
		printd (DEBUG, "group combo index = %d", grp_index);
	}
	else  //no write permission, no changes allowed, just show current values
    {
		*permission=FALSE;	//signal not ok to proceed, even if ok is clicked
    	if ((pw_buf = getpwuid (statbuf.st_uid)) != NULL)
	    	gtk_combo_box_append_text (GTK_COMBO_BOX (user_combo), g_strdup (pw_buf->pw_name));
		else
	 	{
			g_snprintf (id_buf, sizeof (id_buf), "%d", (guint) statbuf.st_uid);
			gtk_combo_box_append_text (GTK_COMBO_BOX (user_combo), id_buf);
		}
    	if ((grp_buf = getgrgid (statbuf.st_gid)) != NULL)
			gtk_combo_box_append_text (GTK_COMBO_BOX (group_combo), g_strdup (grp_buf->gr_name));
		else
		{
			g_snprintf (id_buf, sizeof (id_buf), "%d", (guint) statbuf.st_gid);
			gtk_combo_box_append_text (GTK_COMBO_BOX (group_combo), id_buf);
		}
		gtk_widget_set_sensitive (user_combo, FALSE);
		gtk_widget_set_sensitive (group_combo, FALSE);
	}
    gtk_combo_box_set_active (GTK_COMBO_BOX (user_combo), user_index);
    gtk_combo_box_set_active (GTK_COMBO_BOX (group_combo), grp_index);

    setpwent ();
    setgrent ();
  }

  if (*permission && thisis_dir)  //note: no recurse button now if multiple-selection
  {
 	 table = add_table (dialog_vbox, 1, 3, TRUE, TRUE, 0); //1 row, 3 cols, homogen, fill, no pad
  	gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
  	recurse_dirs_button = e2_button_add_toggle_to_table (table, _("Recurse"), 
	  FALSE, _e2_ownership_dialog_toggle_recurse_button_cb,
	  NULL, 1, 2, 0, 1 ); // gint left, gint right, gint top, gint bottom
  }

  if (!*permission)
	e2_ownership_dialog_warn (dialog_vbox);
  
// add buttons in the order that they will appear
  if (multisrc)
	{
		e2_dialog_add_defined_button (ownership_dialog, &E2_BUTTON_NOTOALL);
		if (*permission)
			e2_dialog_add_button_custom ( ownership_dialog, FALSE, &E2_BUTTON_APPLYTOALL,
			NULL, _e2_ownership_dialog_ok_cb, NULL);
	}
	if (*permission)
		e2_dialog_add_defined_button (ownership_dialog, &E2_BUTTON_CANCEL);
	e2_dialog_add_button_custom (ownership_dialog, TRUE, &E2_BUTTON_OK,
	NULL, _e2_ownership_dialog_ok_cb, NULL);

	e2_dialog_setup (ownership_dialog, app.main_window);
	return ownership_dialog;
}
